package com.gic.haoban.manage.service.service.impl;

import com.gic.api.base.commons.Page;
import com.gic.commons.util.ToolUtil;
import com.gic.haoban.base.api.common.BasePageInfo;
import com.gic.haoban.common.utils.PageUtil;
import com.gic.haoban.manage.api.dto.StoreOrGroupInfoDTO;
import com.gic.haoban.manage.api.enums.ChannelCodeEnum;
import com.gic.haoban.manage.service.dao.mapper.TabHaobanStoreRangeMapper;
import com.gic.haoban.manage.service.dao.mapper.TabHaobanStoreRelationMapper;
import com.gic.haoban.manage.service.entity.TabStoreRange;
import com.gic.haoban.manage.service.entity.TabStoreRelation;
import com.gic.haoban.manage.service.service.StaffClerkRelationService;
import com.gic.haoban.manage.service.service.StoreRangeService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Sets;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

import static org.slf4j.LoggerFactory.getLogger;

/**
 * Created 2021/4/9.
 *
 * @author hua
 */
@Service
public class StoreRangeServiceImpl implements StoreRangeService {

    private static final Logger logger = getLogger(StoreRangeServiceImpl.class);

    @Autowired
    private TabHaobanStoreRangeMapper tabHaobanStoreRangeMapper;

    @Autowired
    private TabHaobanStoreRelationMapper tabHaobanStoreRelationMapper;

    @Autowired
    private StaffClerkRelationService staffClerkRelationService;


    @Override
    public boolean saveStoreRange(List<StoreOrGroupInfoDTO> relations, String enterpriseId, String wxEnterpriseId) {
        tabHaobanStoreRangeMapper.deleteByEnterpriseId(wxEnterpriseId, enterpriseId);
        List<TabStoreRange> ranges = relations.stream().map(dto -> {
            TabStoreRange tabStoreRange = new TabStoreRange();
            tabStoreRange.setEnterpriseId(enterpriseId);
            tabStoreRange.setWxEnterpriseId(wxEnterpriseId);
            tabStoreRange.setRelationId(dto.getRelationId());
            tabStoreRange.setRelationType(dto.getRelationType());
            tabStoreRange.setStoreRangeId(ToolUtil.randomUUID());
            return tabStoreRange;
        }).collect(Collectors.toList());
        tabHaobanStoreRangeMapper.insertBatch(ranges);
        return true;
    }

    @Override
    public List<StoreOrGroupInfoDTO> queryBindStoreRange(String enterpriseId, String wxEnterpriseId) {
        List<TabStoreRange> tabStoreRanges = tabHaobanStoreRangeMapper.queryByEnterpriseId(wxEnterpriseId, enterpriseId);
        if (CollectionUtils.isEmpty(tabStoreRanges)) {
            return null;
        }
        return tabStoreRanges.stream().map(tab -> {
            StoreOrGroupInfoDTO infoDTO = new StoreOrGroupInfoDTO();
            infoDTO.setRelationId(tab.getRelationId());
            infoDTO.setRelationType(tab.getRelationType());
            return infoDTO;
        }).collect(Collectors.toList());
    }

    @Override
    public List<TabStoreRange> queryAllBindRangeByEnterpriseId(String enterpriseId) {
        List<TabStoreRange> tabStoreRanges = tabHaobanStoreRangeMapper.queryByEnterpriseId(null, enterpriseId);
        if (CollectionUtils.isEmpty(tabStoreRanges)) {
            return null;
        }
        return tabStoreRanges;
    }

    @Override
    public boolean delStoreRangeByRelationId(String enterpriseId, String relationId) {
        tabHaobanStoreRangeMapper.delStoreRangeByRelationId(relationId, enterpriseId);
        return true;
    }

    @Override
    public boolean delStoreRelationByStoreId(String enterpriseId, String storeId) {
        tabHaobanStoreRelationMapper.deleteStoreRalation(enterpriseId, storeId);
        return true;
    }

    @Override
    public Page<TabStoreRelation> pageStoreRelation(String enterpriseId, String wxEnterpriseId, String search, BasePageInfo pageInfo) {
        PageHelper.startPage(pageInfo);
        List<TabStoreRelation> storeRelations = tabHaobanStoreRelationMapper.searchStoreRange(wxEnterpriseId, enterpriseId, search);
        Page<TabStoreRelation> retPage = PageUtil.changePageHelperToCurrentPage(new PageInfo<>(storeRelations));
        return retPage;
    }

    @Override
    public boolean saveStoreRelations(List<TabStoreRelation> list, String optStaffId, int channelCode) {
        if (CollectionUtils.isEmpty(list)) {
            logger.info("没有需要保存的");
            return true;
        }
        String enterpriseId = list.get(0).getEnterpriseId();
        //根据wxEnterpriseId 分组
        Map<String, Set<String>> newStoreIdBywxEnterpriseMap = list.stream().collect(Collectors.groupingBy(TabStoreRelation::getWxEnterpriseId, Collectors.mapping(dto -> dto.getStoreId(), Collectors.toSet())));

        List<TabStoreRelation> storeRelations = tabHaobanStoreRelationMapper.listStoreRange(enterpriseId);
        Map<String, Set<String>> oldStoreIdBywxEnterpriseMap = storeRelations.stream().collect(Collectors.groupingBy(TabStoreRelation::getWxEnterpriseId, Collectors.mapping(dto -> dto.getStoreId(), Collectors.toSet())));

        Set<String> needUnBindStoreIds = new HashSet<>();
        oldStoreIdBywxEnterpriseMap.forEach((wxEid, oldStoreIds) -> {
            Set<String> newStoreIds = newStoreIdBywxEnterpriseMap.get(wxEid);
            if (CollectionUtils.isEmpty(newStoreIds)) {
                return;
            }
            Sets.SetView<String> needDelStoreIds = Sets.difference(oldStoreIds, newStoreIds);
            Set<String> mid = needDelStoreIds.stream().collect(Collectors.toSet());
            needUnBindStoreIds.addAll(mid);
        });
        //todo 销售线索 解绑 这些门店解除绑定
        staffClerkRelationService.delBindByStoreIds(null, needUnBindStoreIds, optStaffId, channelCode);
        //先删除 后新增
        tabHaobanStoreRelationMapper.deleteAllStoreRalation(enterpriseId);
        tabHaobanStoreRelationMapper.insertBatch(list);
        return true;
    }

    @Override
    public List<TabStoreRange> queryStoreRangeByRelationId(String enterpriseId, List<String> relationIds) {
        return tabHaobanStoreRangeMapper.listStoreRangeByRelationId(relationIds, enterpriseId);
    }
}
